// script.js

// @flow
// NotePlan Plugin to generate the habit tracker and update the daily note.

/**
 * Main function to update the habit tracker.
 */
async function updateHabitTracker() {
  try {
    // Fetch the "logs data" note
    const logsDataNote = DataStore.projectNotes.find(
      (note) => note.title === 'logs data' || note.filename === 'logsdata.txt'
    );

    if (!logsDataNote) {
      await CommandBar.prompt(
        'Error',
        'The "logs data" note was not found. Please ensure it exists.'
      );
      return;
    }

    // Read the content of the "logs data" note
    const logData = logsDataNote.content || '';

    // Parse the log entries
    const logEntries = logData.trim().split('\n');

    // Initialize data structures
    const dailyTracker = {}; // { date: { habit: true } }
    const allHabits = new Set();

    // Get current date information
    const today = new Date();
    const currentMonth = today.getMonth(); // 0-indexed
    const currentYear = today.getFullYear();
    const currentMonthYear = today.toLocaleDateString('default', {
      month: 'long',
      year: 'numeric',
    });

    // Get all dates in the current month
    const monthDates = getDatesInMonth(currentYear, currentMonth);

    // Process each log entry
    for (const entry of logEntries) {
      // Split the entry by '-,'
      const parts = entry.split('-,').map((part) => part.trim());

      if (parts.length < 2) {
        // Malformed entry; skip and log the issue
        console.log(`Skipping invalid entry: ${entry}`);
        continue;
      }

      const dateTimeStr = parts[0];
      let habit = parts[1];

      // Clean up the habit name by removing any extra characters
      habit = habit.replace(/-,-,/g, '').trim();

      // Extract the date part from dateTimeStr
      const dateMatch = dateTimeStr.match(/^(.+?) at /);
      if (!dateMatch) {
        console.log(`Invalid date format in entry: ${entry}`);
        continue;
      }

      const dateStr = dateMatch[1].trim();

      // Parse the date
      const dateObj = parseDate(dateStr);

      if (!dateObj) {
        console.log(`Invalid date format in entry: ${entry}`);
        continue;
      }

      // Only consider entries from the current month and year
      if (
        dateObj.getMonth() !== currentMonth ||
        dateObj.getFullYear() !== currentYear
      )
        continue;

      const dateISO = formatDate(dateObj);

      // Initialize date in tracker if not present
      if (!dailyTracker[dateISO]) dailyTracker[dateISO] = {};

      // Mark the habit as completed on that date
      dailyTracker[dateISO][habit] = true;
      allHabits.add(habit);
    }

    // Sort habits alphabetically
    const sortedHabits = Array.from(allHabits).sort();

    // Generate the habit tracker content
    const trackerContent = generateTrackerContent(
      currentMonthYear,
      monthDates,
      today,
      sortedHabits,
      dailyTracker
    );

    // Create or overwrite the "Habit Tracker for [Month Year]" note
    const trackerNoteTitle = `Habit Tracker for ${currentMonthYear}`;
    let trackerNote = DataStore.projectNotes.find(
      (note) => note.title === trackerNoteTitle
    );

    // Save the habit tracker note in the same folder as the "logs data" note
    const logsDataFolder = logsDataNote.filename
      .split('/')
      .slice(0, -1)
      .join('/');

    if (!trackerNote) {
      // Create a new note in the same folder as "logs data"
      trackerNote = await DataStore.newNote(
        trackerContent,
        trackerNoteTitle,
        logsDataFolder
      );
    } else {
      // Overwrite existing note
      trackerNote.content = trackerContent;
    }

    // Extract the summary before the '---' separator
    const summaryContent = trackerContent.split('---')[0].trim();

    // Update today's daily note
    const dailyNote = DataStore.calendarNoteByDate(today);

    if (!dailyNote) {
      await CommandBar.prompt(
        'Error',
        "Today's daily note could not be opened."
      );
      return;
    }

    // Prepend the summary under the heading "Logged Habits for today:"
    const heading = 'Logged Habits for today:';
    let dailyContent = dailyNote.content || '';

    // Construct the new content
    const headingString = `## ${heading}`;
    const summaryWithHeading = `${headingString}\n${summaryContent}\n`;

    // Check if the heading already exists
    const regex = new RegExp(`(${headingString})([\\s\\S]*?)(##|$)`, 'm');
    if (regex.test(dailyContent)) {
      // Replace existing content under the heading
      dailyContent = dailyContent.replace(regex, `${summaryWithHeading}$3`);
    } else {
      // Prepend the heading and summary
      dailyContent = `${summaryWithHeading}\n${dailyContent}`;
    }

    // Update the daily note content
    dailyNote.content = dailyContent;

    await CommandBar.prompt(
      'Success',
      'Habit tracker updated successfully.'
    );
  } catch (error) {
    console.log(`Error in updateHabitTracker: ${error}`);
    await CommandBar.prompt(
      'Error',
      'An error occurred while updating the habit tracker.'
    );
  }
}

/**
 * Generates the content for the habit tracker note.
 */
function generateTrackerContent(
  currentMonthYear,
  monthDates,
  today,
  sortedHabits,
  dailyTracker
) {
  let content = `### Habit Tracker for ${currentMonthYear}\n[[Habit Tracker for ${currentMonthYear}]]\n`;
  content += '```markdown\n';

  // Determine the maximum habit name length for alignment
  const maxHabitLength = Math.max(...sortedHabits.map((h) => h.length));

  // Use habit names padded to the maximum length
  for (const habit of sortedHabits) {
    let progressBar = '';
    for (const dateObj of monthDates) {
      const dateISO = formatDate(dateObj);

      if (dateObj <= today) {
        if (dailyTracker[dateISO] && dailyTracker[dateISO][habit]) {
          progressBar += '🟩'; // Habit logged
        } else {
          progressBar += '🟥'; // Habit not logged
        }
      } else {
        progressBar += '⬜️'; // Future date
      }
    }
    const habitNamePadded = habit.padEnd(maxHabitLength);
    content += `${habitNamePadded}  ${progressBar}\n`;
  }
  content += '```\n\n---\n\n';

  // Generate detailed table
  content += `| Date       | ${sortedHabits.join(' | ')} |\n`;
  content += `|------------|${sortedHabits
    .map(() => '------------')
    .join('|')}|\n`;

  for (const dateObj of monthDates) {
    const dateISO = formatDate(dateObj);
    const row = sortedHabits.map((habit) => {
      if (dailyTracker[dateISO] && dailyTracker[dateISO][habit]) {
        return '✅';
      } else {
        return ' ';
      }
    });
    content += `| ${dateISO} | ${row.join(' | ')} |\n`;
  }

  return content;
}

/**
 * Helper function to get all dates in a specific month.
 */
function getDatesInMonth(year, month) {
  const date = new Date(year, month, 1);
  const dates = [];

  while (date.getMonth() === month) {
    dates.push(new Date(date));
    date.setDate(date.getDate() + 1);
  }

  return dates;
}

/**
 * Helper function to format date as YYYY-MM-DD.
 */
function formatDate(dateObj) {
  const year = dateObj.getFullYear();
  const month = String(dateObj.getMonth() + 1).padStart(2, '0');
  const day = String(dateObj.getDate()).padStart(2, '0');
  return `${year}-${month}-${day}`;
}

/**
 * Helper function to parse date strings like 'Oct 19, 2024'.
 */
function parseDate(dateStr) {
  const dateParts = dateStr.match(/(\w+) (\d+), (\d+)/);
  if (!dateParts) return null;

  const monthNames = [
    'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun',
    'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec',
  ];
  const monthIndex = monthNames.indexOf(dateParts[1]);
  if (monthIndex === -1) return null;

  const day = parseInt(dateParts[2], 10);
  const year = parseInt(dateParts[3], 10);

  return new Date(year, monthIndex, day);
}

/**
 * Function to launch the Apple Shortcut.
 */
function launchShortcut() {
  try {
    const url = 'shortcuts://run-shortcut?name=NotePlan%20Log%20Item';
    NotePlan.openURL(url);
  } catch (error) {
    console.log(`Error in launchShortcut: ${error}`);
    CommandBar.prompt(
      'Error',
      'An error occurred while launching the shortcut.'
    );
  }
}

// Expose functions to NotePlan
globalThis.updateHabitTracker = updateHabitTracker;
globalThis.launchShortcut = launchShortcut;